@@ -21,9 +21,18 @@
 #include "ColordKCM.h"
 #include "ui_ColordKCM.h"
 
+#include <config.h>
+
 #include "DeviceModel.h"
 #include "ProfileModel.h"
 #include "Description.h"
+#include "NoSelectionRectDelegate.h"
+
+#include "CdInterface.h"
+#include "CdProfileInterface.h"
+#include "CdDeviceInterface.h"
+
+#include <KProcess>
 
 #include <KMessageBox>
 #include <KGenericFactory>
@@ -33,9 +42,6 @@
 #include <KIcon>
 #include <KUser>
 
-#include <QtDBus/QDBusInterface>
-#include <QtDBus/QDBusConnection>
-#include <QtDBus/QDBusMessage>
 #include <QtDBus/QDBusServiceWatcher>
 #include <QItemSelectionModel>
 #include <QTimer>
@@ -56,12 +62,13 @@ ColordKCM::ColordKCM(QWidget *parent, const QVariantList &args) :
     aboutData = new KAboutData("kcm_colord",
                                "kcm_colord",
                                ki18n("Color settings"),
-                               "0.2",
+                               COLORD_KDE_VERSION,
                                ki18n("Color settings"),
                                KAboutData::License_GPL,
-                               ki18n("(C) 2012 Daniel Nicoletti"));
+                               ki18n("(C) 2012-2013 Daniel Nicoletti"));
     setAboutData(aboutData);
     setButtons(NoAdditionalButton);
+    KGlobal::insertCatalog(QLatin1String("colord-kde"));
 
     ui->setupUi(this);
     ui->infoWidget->setPixmap(KTitleWidget::InfoMessage);
@@ -89,6 +96,18 @@ ColordKCM::ColordKCM(QWidget *parent, const QVariantList &args) :
     connect(ui->removeProfileBt, SIGNAL(clicked()),
             this, SLOT(removeProfile()));
 
+    // Creates a ColorD interface, it must be created with new
+    // otherwise the object will be deleted when this block ends
+    m_cdInterface = new CdInterface(QLatin1String("org.freedesktop.ColorManager"),
+                                    QLatin1String("/org/freedesktop/ColorManager"),
+                                    QDBusConnection::systemBus(),
+                                    this);
+    ui->profile->setCdInterface(m_cdInterface);
+
+    // listen to colord for events
+    connect(m_cdInterface, SIGNAL(ProfileAdded(QDBusObjectPath)),
+            this, SLOT(profileAdded(QDBusObjectPath)));
+
     // Devices view setup
     QSortFilterProxyModel *sortModel = new QSortFilterProxyModel(this);
     // Connect this slot prior to defining the model
@@ -100,15 +119,16 @@ ColordKCM::ColordKCM(QWidget *parent, const QVariantList &args) :
     sortModel->sort(0);
     // Set the source model then connect to the selection model to get updates
     ui->devicesTV->setModel(sortModel);
+    ui->devicesTV->setItemDelegate(new NoSelectionRectDelegate(this));
     connect(ui->devicesTV->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
             this, SLOT(showDescription()));
 
-    m_deviceModel = new DeviceModel(this);
+    m_deviceModel = new DeviceModel(m_cdInterface, this);
     connect(m_deviceModel, SIGNAL(changed()), this, SLOT(updateSelection()));
     sortModel->setSourceModel(m_deviceModel);
 
     // Profiles view setup
-    m_profileModel = new ProfileModel(this);
+    m_profileModel = new ProfileModel(m_cdInterface, this);
     connect(m_profileModel, SIGNAL(changed()), this, SLOT(updateSelection()));
     // Filter Proxy for the menu
     m_profilesFilter = new QSortFilterProxyModel(this);
@@ -125,6 +145,7 @@ ColordKCM::ColordKCM(QWidget *parent, const QVariantList &args) :
     profileSortModel->setSortRole(ProfileModel::SortRole);
     profileSortModel->sort(0);
     ui->profilesTV->setModel(profileSortModel);
+    ui->profilesTV->setItemDelegate(new NoSelectionRectDelegate(this));
     connect(ui->profilesTV->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
             this, SLOT(showDescription()));
     connect(profileSortModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
@@ -144,19 +165,6 @@ ColordKCM::ColordKCM(QWidget *parent, const QVariantList &args) :
     connect(watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
             ui->profile, SLOT(serviceOwnerChanged(QString,QString,QString)));
 
-    // Creates a ColorD interface, it must be created with new
-    // otherwise the object will be deleted when this block ends
-    QDBusInterface *interface;
-    interface = new QDBusInterface(QLatin1String("org.freedesktop.ColorManager"),
-                                   QLatin1String("/org/freedesktop/ColorManager"),
-                                   QLatin1String("org.freedesktop.ColorManager"),
-                                   QDBusConnection::systemBus(),
-                                   this);
-
-    // listen to colord for events
-    connect(interface, SIGNAL(ProfileAdded(QDBusObjectPath)),
-            this, SLOT(profileAdded(QDBusObjectPath)));
-
     ui->devicesTb->setIcon(KIcon("preferences-activities"));
     ui->profilesTb->setIcon(KIcon("application-vnd.iccprofile"));
 
@@ -208,20 +216,18 @@ void ColordKCM::showDescription()
         return;
     }
 
+    bool canRemoveProfile = index.data(ProfileModel::CanRemoveProfileRole).toBool();
     if (index.data(DeviceModel::IsDeviceRole).toBool()) {
         ui->profile->setDevice(index.data(ProfileModel::ObjectPathRole).value<QDBusObjectPath>());
     } else {
-        ui->profile->setProfile(index.data(ProfileModel::ObjectPathRole).value<QDBusObjectPath>());
+        ui->profile->setProfile(index.data(ProfileModel::ObjectPathRole).value<QDBusObjectPath>(),
+                                canRemoveProfile);
     }
+    ui->removeProfileBt->setEnabled(canRemoveProfile);
 
     if (ui->stackedWidget->currentWidget() != ui->profile_page) {
         ui->stackedWidget->setCurrentWidget(ui->profile_page);
     }
-
-    // Check if we can remove the Profile
-    bool enable;
-    enable = index.data(ProfileModel::CanRemoveProfileRole).toBool();
-    ui->removeProfileBt->setEnabled(enable);
 }
 
 void ColordKCM::addProfileFile()
@@ -236,19 +242,23 @@ void ColordKCM::addProfileFile()
         return;
     }
 
+    // Store the device kind and device object path
+    // so that we assign the profile to the device when
+    // ProfileAdded is emitted
     QFileInfo fileInfo(fileName);
+    QString kind = index.data(DeviceModel::ProfileKindRole).toString();
     QString newFilename = profilesPath() % fileInfo.fileName();
-    if (!QFile::copy(fileName, newFilename)) {
-        KMessageBox::sorry(this, i18n("Failed to import color profile"), i18n("Importing Color Profile"));
-    } else if (index.isValid()) {
-        // Store the device kind and device object path
-        // so that we assign the profile to the device when
-        // ProfileAdded is emitted
-        QString kind;
-        QDBusObjectPath devicePath;
-        kind = index.data(DeviceModel::ProfileKindRole).toString();
-        devicePath = index.data(DeviceModel::ObjectPathRole).value<QDBusObjectPath>();
-        m_profileFiles[newFilename] = KindAndPath(kind, devicePath);
+    QDBusObjectPath devicePath;
+    devicePath = index.data(DeviceModel::ObjectPathRole).value<QDBusObjectPath>();
+    m_profileFiles[newFilename] = KindAndPath(kind, devicePath);
+
+    KProcess process;
+    process << QLatin1String("colord-kde-icc-importer");
+    process << QLatin1String("--yes");
+    process << fileName;
+    process.execute();
+    if (process.exitStatus() != QProcess::NormalExit || process.exitCode()) {
+        m_profileFiles.remove(newFilename);
     }
 }
 
@@ -299,15 +309,12 @@ void ColordKCM::removeProfile()
         deviceObject  = index.data(ProfileModel::ParentObjectPathRole).value<QDBusObjectPath>();
         profileObject = index.data(ProfileModel::ObjectPathRole).value<QDBusObjectPath>();
 
-        QDBusMessage message;
-        message = QDBusMessage::createMethodCall(QLatin1String("org.freedesktop.ColorManager"),
-                                                 deviceObject.path(),
-                                                 QLatin1String("org.freedesktop.ColorManager.Device"),
-                                                 QLatin1String("RemoveProfile"));
-        message << qVariantFromValue(profileObject); // Profile Path
-
-        /* call Device.RemoveProfile() with the device and profile object paths */
-        QDBusConnection::systemBus().send(message);
+        CdDeviceInterface device(QLatin1String("org.freedesktop.ColorManager"),
+                                 deviceObject.path(),
+                                 QDBusConnection::systemBus());
+        if (device.isValid()) {
+            device.RemoveProfile(profileObject);
+        }
     } else {
         // We need to remove the file
         QString filename = index.data(ProfileModel::FilenameRole).toString();
@@ -384,18 +391,16 @@ void ColordKCM::on_tabWidget_currentChanged(int index)
 
 void ColordKCM::profileAdded(const QDBusObjectPath &objectPath)
 {
-    QDBusInterface interface(QLatin1String("org.freedesktop.ColorManager"),
-                             objectPath.path(),
-                             QLatin1String("org.freedesktop.ColorManager.Profile"),
-                             QDBusConnection::systemBus(),
-                             this);
+    CdProfileInterface profile(QLatin1String("org.freedesktop.ColorManager"),
+                               objectPath.path(),
+                               QDBusConnection::systemBus());
 
-    if (!interface.isValid()) {
+    if (!profile.isValid()) {
         return;
     }
 
-    QString kind = interface.property("Kind").toString();
-    QString filename = interface.property("Filename").toString();
+    QString kind = profile.kind();
+    QString filename = profile.filename();
 
     if (m_profileFiles.contains(filename)) {
         if (m_profileFiles[filename].first != kind) {
@@ -410,18 +415,14 @@ void ColordKCM::profileAdded(const QDBusObjectPath &objectPath)
     }
 }
 
-void ColordKCM::addProvileToDevice(const QDBusObjectPath &profilePath, const QDBusObjectPath &devicePath) const
+void ColordKCM::addProvileToDevice(const QDBusObjectPath &profile, const QDBusObjectPath &devicePath) const
 {
-    QDBusMessage message;
-    message = QDBusMessage::createMethodCall(QLatin1String("org.freedesktop.ColorManager"),
-                                             devicePath.path(),
-                                             QLatin1String("org.freedesktop.ColorManager.Device"),
-                                             QLatin1String("AddProfile"));
-    message << QString("hard"); // Relation
-    message << qVariantFromValue(profilePath); // Profile Path
-
-    /* call Device.AddProfile() with the device and profile object paths */
-    QDBusConnection::systemBus().send(message);
+    CdDeviceInterface device(QLatin1String("org.freedesktop.ColorManager"),
+                             devicePath.path(),
+                             QDBusConnection::systemBus());
+    if (device.isValid()) {
+        device.AddProfile(QLatin1String("hard"), profile);
+    }
 }
 
 QModelIndex ColordKCM::currentIndex() const
@@ -441,12 +442,12 @@ QModelIndex ColordKCM::currentIndex() const
 
         if (ui->tabWidget->currentIndex() == 0) {
             // Devices is empty
-            ui->infoWidget->setText(i18n("You don't have any devices registered"));
+            ui->infoWidget->setText(i18n("You do not have any devices registered"));
             ui->infoWidget->setComment(i18n("Make sure colord module on kded is running"));
         } else {
             // Profiles is empty
-            ui->infoWidget->setText(i18n("You don't have any profiles registered"));
-            ui->infoWidget->setComment(i18n("Add one by clicking add profile buttom"));
+            ui->infoWidget->setText(i18n("You do not have any profiles registered"));
+            ui->infoWidget->setComment(i18n("Add one by clicking Add Profile button"));
         }
 
         return ret;
