@@ -34,13 +34,13 @@ typedef QPair<QString, QDBusObjectPath> KindAndPath;
 namespace Ui {
     class ColordKCM;
 }
+class CdInterface;
 class DeviceModel;
 class ProfileModel;
 class ProfileDescription;
 class ColordKCM : public KCModule
 {
-Q_OBJECT
-
+    Q_OBJECT
 public:
     ColordKCM(QWidget *parent, const QVariantList &args);
     ~ColordKCM();
@@ -59,7 +59,7 @@ private slots:
     void profileAdded(const QDBusObjectPath &objectPath);
 
 private:
-    void addProvileToDevice(const QDBusObjectPath &profilePath, const QDBusObjectPath &devicePath) const;
+    void addProvileToDevice(const QDBusObjectPath &profile, const QDBusObjectPath &devicePath) const;
     QModelIndex currentIndex() const;
     QString profilesPath() const;
 
@@ -80,6 +80,7 @@ private:
     QAction *m_addAction;
     QAction *m_removeAction;
     QAction *m_configureAction;
+    CdInterface *m_cdInterface;
 };
 
 #endif // COLORD_KCM_H
