@@ -24,18 +24,22 @@
 #include "ProfileNamedColors.h"
 #include "ProfileMetaData.h"
 
+#include "CdInterface.h"
+#include "CdDeviceInterface.h"
+#include "CdProfileInterface.h"
+#include "CdSensorInterface.h"
+
 #include <math.h>
 
 #include <QFileInfo>
 #include <QStringBuilder>
-#include <QtDBus/QDBusInterface>
-#include <QtDBus/QDBusReply>
 
 #include <KGlobal>
 #include <KLocale>
 #include <KDateTime>
 #include <KToolInvocation>
 #include <KDebug>
+#include <KMessageWidget>
 
 #define TAB_INFORMATION  1
 #define TAB_CIE_1931     2
@@ -47,38 +51,19 @@
 #define TAB_METADATA     8
 
 typedef QList<QDBusObjectPath> ObjectPathList;
-typedef QMap<QString, QString>  StringStringMap;
 
 Description::Description(QWidget *parent) :
     QWidget(parent),
     ui(new Ui::Description)
 {
     ui->setupUi(this);
+    ui->msgWidget->setMessageType(KMessageWidget::Warning);
+    ui->msgWidget->setWordWrap(true);
+    ui->msgWidget->setCloseButtonVisible(false);
+    ui->msgWidget->hide();
 
     m_namedColors = new ProfileNamedColors;
     m_metadata = new ProfileMetaData;
-
-    // Creates a ColorD interface, it must be created with new
-    // otherwise the object will be deleted when this block ends
-    QDBusInterface *interface;
-    interface = new QDBusInterface(QLatin1String("org.freedesktop.ColorManager"),
-                                   QLatin1String("/org/freedesktop/ColorManager"),
-                                   QLatin1String("org.freedesktop.ColorManager"),
-                                   QDBusConnection::systemBus(),
-                                   this);
-    // listen to colord for events
-    connect(interface, SIGNAL(SensorAdded(QDBusObjectPath)),
-            this, SLOT(sensorAdded(QDBusObjectPath)));
-    connect(interface, SIGNAL(SensorRemoved(QDBusObjectPath)),
-            this, SLOT(sensorRemoved(QDBusObjectPath)));
-
-    // Ask for profiles
-    QDBusMessage message;
-    message = QDBusMessage::createMethodCall(QLatin1String("org.freedesktop.ColorManager"),
-                                             QLatin1String("/org/freedesktop/ColorManager"),
-                                             QLatin1String("org.freedesktop.ColorManager"),
-                                             QLatin1String("GetSensors"));
-    QDBusConnection::systemBus().callWithCallback(message, this, SLOT(gotSensors(QDBusMessage)));
 }
 
 Description::~Description()
@@ -94,27 +79,38 @@ int Description::innerHeight() const
     return ui->tabWidget->currentWidget()->height();
 }
 
-void Description::setProfile(const QDBusObjectPath &objectPath)
+void Description::setCdInterface(CdInterface *interface)
+{
+    // listen to colord for events
+    connect(interface, SIGNAL(SensorAdded(QDBusObjectPath)),
+            this, SLOT(sensorAdded(QDBusObjectPath)));
+    connect(interface, SIGNAL(SensorRemoved(QDBusObjectPath)),
+            this, SLOT(sensorRemoved(QDBusObjectPath)));
+
+    QDBusPendingReply<ObjectPathList> async = interface->GetSensors();
+    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(async, this);
+    connect(watcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
+            this, SLOT(gotSensors(QDBusPendingCallWatcher*)));
+}
+
+void Description::setProfile(const QDBusObjectPath &objectPath, bool canRemoveProfile)
 {
     m_currentProfile = objectPath;
     m_currentDeviceId.clear();
 
     ui->stackedWidget->setCurrentIndex(0);
-    QDBusInterface profileInterface(QLatin1String("org.freedesktop.ColorManager"),
-                                    objectPath.path(),
-                                    QLatin1String("org.freedesktop.ColorManager.Profile"),
-                                    QDBusConnection::systemBus(),
-                                    this);
+    CdProfileInterface profileInterface(QLatin1String("org.freedesktop.ColorManager"),
+                                        objectPath.path(),
+                                        QDBusConnection::systemBus());
     if (!profileInterface.isValid()) {
         return;
     }
 
-    QString filename = profileInterface.property("Filename").toString();
-    bool hasVcgt = profileInterface.property("HasVcgt").toBool();
-    bool isSystemWide = profileInterface.property("IsSystemWide").toBool();
-    qulonglong created = profileInterface.property("Created").toULongLong();
+    QString filename = profileInterface.filename();
+    bool hasVcgt = profileInterface.hasVcgt();
+    qulonglong created = profileInterface.created();
 
-    ui->installSystemWideBt->setEnabled(!isSystemWide);
+    ui->installSystemWideBt->setEnabled(canRemoveProfile);
     Profile profile(filename);
     if (profile.loaded()) {
         // Set the profile type
@@ -136,7 +132,7 @@ void Description::setProfile(const QDBusObjectPath &objectPath)
         ui->licenseL->setVisible(!profile.copyright().isEmpty());
         ui->licenseLabel->setVisible(!profile.copyright().isEmpty());
 
-        // Set the manufaturer
+        // Set the manufacturer
         ui->deviceMakeL->setText(profile.manufacturer());
         ui->deviceMakeL->setVisible(!profile.manufacturer().isEmpty());
         ui->makeLabel->setVisible(!profile.manufacturer().isEmpty());
@@ -181,22 +177,7 @@ void Description::setProfile(const QDBusObjectPath &objectPath)
             removeTab(m_namedColors);
         }
 
-        // Get profile metadata
-        QDBusMessage message;
-        message = QDBusMessage::createMethodCall(QLatin1String("org.freedesktop.ColorManager"),
-                                                 objectPath.path(),
-                                                 QLatin1String("org.freedesktop.DBus.Properties"),
-                                                 QLatin1String("Get"));
-        message << QString("org.freedesktop.ColorManager.Profile"); // Interface
-        message << QString("Metadata"); // Propertie Name
-        QDBusReply<QVariant> reply = QDBusConnection::systemBus().call(message, QDBus::BlockWithGui);
-
-        StringStringMap metadata;
-        if (reply.isValid()) {
-            QDBusArgument argument = reply.value().value<QDBusArgument>();
-            metadata = qdbus_cast<StringStringMap>(argument);
-        }
-
+        CdStringMap metadata = profileInterface.metadata();
         if (!metadata.isEmpty()) {
             m_metadata->setMetadata(metadata);
             insertTab(TAB_METADATA, m_metadata, i18n("Metadata"));
@@ -215,22 +196,20 @@ void Description::setDevice(const QDBusObjectPath &objectPath)
 
     ui->stackedWidget->setCurrentIndex(1);
 
-    QDBusInterface deviceInterface(QLatin1String("org.freedesktop.ColorManager"),
-                                   objectPath.path(),
-                                   QLatin1String("org.freedesktop.ColorManager.Device"),
-                                   QDBusConnection::systemBus(),
-                                   this);
-    if (!deviceInterface.isValid()) {
+    CdDeviceInterface device(QLatin1String("org.freedesktop.ColorManager"),
+                             objectPath.path(),
+                             QDBusConnection::systemBus());
+    if (!device.isValid()) {
         return;
     }
 
     QString deviceTitle;
-    m_currentDeviceId = deviceInterface.property("DeviceId").toString();
-    QString kind = deviceInterface.property("Kind").toString();
+    m_currentDeviceId = device.deviceId();
+    QString kind = device.kind();
     m_currentDeviceKind = kind;
-    QString model = deviceInterface.property("Model").toString();
-    QString vendor = deviceInterface.property("Vendor").toString();
-    QString scope = deviceInterface.property("Scope").toString();
+    QString model = device.model();
+    QString vendor = device.vendor();
+    QString scope = device.scope();
     if (model.isEmpty() && vendor.isEmpty()) {
         deviceTitle = m_currentDeviceId;
     } else if (model.isEmpty()) {
@@ -243,71 +222,67 @@ void Description::setDevice(const QDBusObjectPath &objectPath)
     ui->ktitlewidget->setText(deviceTitle);
 
     if (kind == QLatin1String("printer")) {
-        kind = i18n("Printer");
+        kind = i18nc("device type", "Printer");
     } else if (kind == QLatin1String("display")) {
-        kind = i18n("Display");
+        kind = i18nc("device type", "Display");
     } else if (kind == QLatin1String("webcam")) {
-        kind = i18n("Webcam");
+        kind = i18nc("device type", "Webcam");
     } else if (kind == QLatin1String("scanner")) {
-        kind = i18n("Scanner");
+        kind = i18nc("device type", "Scanner");
     } else {
-        kind = i18n("Unknown");
+        kind = i18nc("device type", "Unknown");
     }
     ui->ktitlewidget->setComment(kind);
 
     ui->deviceIdL->setText(m_currentDeviceId);
 
     if (scope == QLatin1String("temp")) {
-        scope = i18n("User session");
+        scope = i18nc("device scope", "User session");
     } else if (scope == QLatin1String("disk")) {
-        scope = i18n("Auto restore");
+        scope = i18nc("device scope", "Auto restore");
     } else if (scope == QLatin1String("normal")) {
-        scope = i18n("System wide");
+        scope = i18nc("device scope", "System wide");
     } else {
-        scope = i18n("Unknown");
+        scope = i18nc("device scope", "Unknown");
     }
     ui->deviceScopeL->setText(scope);
 
-    QString colorspace = deviceInterface.property("Colorspace").toString();
+    QString colorspace = device.colorspace();
     if (colorspace == QLatin1String("rgb")) {
-        colorspace = i18n("RGB");
+        colorspace = i18nc("colorspace", "RGB");
     } else if (colorspace == QLatin1String("cmyk")) {
-        colorspace = i18n("CMYK");
+        colorspace = i18nc("colorspace", "CMYK");
     } else if (colorspace == QLatin1String("gray")) {
-        colorspace = i18n("Gray");
+        colorspace = i18nc("colorspace", "Gray");
     }
     ui->modeL->setText(colorspace);
 
-    ObjectPathList profiles = deviceInterface.property("Profiles").value<ObjectPathList>();
+    ObjectPathList profiles = device.profiles();
 
     QString profileTitle = i18n("This device has no profile assigned to it");
     if (!profiles.isEmpty()) {
-        QDBusInterface profileInterface(QLatin1String("org.freedesktop.ColorManager"),
-                                        profiles.first().path(),
-                                        QLatin1String("org.freedesktop.ColorManager.Profile"),
-                                        QDBusConnection::systemBus(),
-                                        this);
-        if (profileInterface.isValid()) {
-            profileTitle = profileInterface.property("Title").toString();
+        CdProfileInterface profile(QLatin1String("org.freedesktop.ColorManager"),
+                                   profiles.first().path(),
+                                   QDBusConnection::systemBus());
+        if (profile.isValid()) {
+            profileTitle = profile.title();
             if (profileTitle.isEmpty()) {
-                profileTitle = profileInterface.property("ProfileId").toString();
+                profileTitle = profile.profileId();
             }
         }
     }
     ui->defaultProfileName->setText(profileTitle);
 
-    // Verify if the Calibrate button should be enbled or disabled
+    // Verify if the Calibrate button should be enabled or disabled
     ui->calibratePB->setEnabled(calibrateEnabled(m_currentDeviceKind));
 }
 
 void Description::on_installSystemWideBt_clicked()
 {
-    QDBusMessage message;
-    message = QDBusMessage::createMethodCall(QLatin1String("org.freedesktop.ColorManager"),
-                                             m_currentProfile.path(),
-                                             QLatin1String("org.freedesktop.ColorManager.Profile"),
-                                             QLatin1String("InstallSystemWide"));
-    QDBusConnection::systemBus().send(message);
+    CdProfileInterface profile(QLatin1String("org.freedesktop.ColorManager"),
+                               m_currentProfile.path(),
+                               QDBusConnection::systemBus());
+    profile.InstallSystemWide();
 }
 
 void Description::on_calibratePB_clicked()
@@ -321,19 +296,20 @@ void Description::on_calibratePB_clicked()
     KToolInvocation::kdeinitExec(QLatin1String("gcm-calibrate"), args);
 }
 
-void Description::gotSensors(const QDBusMessage &message)
+void Description::gotSensors(QDBusPendingCallWatcher *call)
 {
-    if (message.type() == QDBusMessage::ReplyMessage && message.arguments().size() == 1) {
-        QDBusArgument argument = message.arguments().first().value<QDBusArgument>();
-        ObjectPathList paths = qdbus_cast<ObjectPathList>(argument);
-        foreach (const QDBusObjectPath &path, paths) {
+    QDBusPendingReply<ObjectPathList> reply = *call;
+    if (reply.isError()) {
+        kWarning() << "Unexpected message" << reply.error().message();
+    } else {
+        ObjectPathList sensors = reply.argumentAt<0>();
+        foreach (const QDBusObjectPath &sensor, sensors) {
             // Add the sensors but don't update the Calibrate button
-            sensorAdded(path, false);
+            sensorAdded(sensor, false);
         }
+
         // Update the calibrate button later
         ui->calibratePB->setEnabled(calibrateEnabled(m_currentDeviceKind));
-    } else {
-        kWarning() << "Unexpected message" << message;
     }
 }
 
@@ -397,7 +373,7 @@ void Description::removeTab(QWidget *widget)
 bool Description::calibrateEnabled(const QString &kind)
 {
     QString toolTip;
-    bool ret = false;
+    bool canCalibrate = false;
     toolTip = i18n("Create a color profile for the selected device");
 
     if (m_currentDeviceId.isEmpty()) {
@@ -413,12 +389,12 @@ bool Description::calibrateEnabled(const QString &kind)
         if (m_sensors.isEmpty()) {
             toolTip = i18n("The measuring instrument is not detected. Please check it is turned on and correctly connected.");
         } else {
-            ret = true;
+            canCalibrate = true;
         }
     } else if (kind == QLatin1String("camera") ||
                kind == QLatin1String("scanner") ||
                kind == QLatin1String("webcam")) {
-        ret = true;
+        canCalibrate = true;
     } else if (kind == QLatin1String("printer")) {
         // Check if we have any sensor
         if (m_sensors.isEmpty()) {
@@ -426,24 +402,22 @@ bool Description::calibrateEnabled(const QString &kind)
         } else {
             // Search for a suitable sensor
             foreach (const QDBusObjectPath &sensorPath, m_sensors) {
-                QDBusInterface sensorInterface(QLatin1String("org.freedesktop.ColorManager"),
-                                               sensorPath.path(),
-                                               QLatin1String("org.freedesktop.ColorManager.Sensor"),
-                                               QDBusConnection::systemBus(),
-                                               this);
-                if (!sensorInterface.isValid()) {
+                CdSensorInterface sensor(QLatin1String("org.freedesktop.ColorManager"),
+                                         sensorPath.path(),
+                                         QDBusConnection::systemBus());
+                if (!sensor.isValid()) {
                     continue;
                 }
 
-                QStringList capabilities = sensorInterface.property("Capabilities").toStringList();
+                QStringList capabilities = sensor.capabilities();
                 if (capabilities.contains(QLatin1String("printer"))) {
-                    ret = true;
+                    canCalibrate = true;
                     break;
                 }
             }
 
-            // If we do not found a suitable sensor place a proper tool tip
-            if (!ret) {
+            // If we did not find a suitable sensor, display a warning
+            if (!canCalibrate) {
                 toolTip = i18n("The measuring instrument does not support printer profiling.");
             }
         }
@@ -451,8 +425,15 @@ bool Description::calibrateEnabled(const QString &kind)
         toolTip = i18n("The device type is not currently supported.");
     }
 
-    ui->calibratePB->setToolTip(toolTip);
-    return ret;
+    if (canCalibrate) {
+        ui->calibratePB->setToolTip(toolTip);
+        ui->msgWidget->hide();
+    } else {
+        ui->msgWidget->setText(toolTip);
+        ui->msgWidget->show();
+    }
+
+    return canCalibrate;
 }
 
 #include "Description.moc"
