@@ -22,11 +22,12 @@
 
 #include <QWidget>
 #include <QDBusObjectPath>
-#include <QDBusMessage>
+#include <QDBusPendingCallWatcher>
 
 namespace Ui {
     class Description;
 }
+class CdInterface;
 class ProfileNamedColors;
 class ProfileMetaData;
 class Description : public QWidget
@@ -37,7 +38,8 @@ public:
     ~Description();
 
     int innerHeight() const;
-    void setProfile(const QDBusObjectPath &objectPath);
+    void setCdInterface(CdInterface *interface);
+    void setProfile(const QDBusObjectPath &objectPath, bool canRemoveProfile);
     void setDevice(const QDBusObjectPath &objectPath);
 
 public slots:
@@ -47,7 +49,7 @@ private slots:
     void on_installSystemWideBt_clicked();
     void on_calibratePB_clicked();
 
-    void gotSensors(const QDBusMessage &message);
+    void gotSensors(QDBusPendingCallWatcher *call);
     void sensorAdded(const QDBusObjectPath &sensorPath, bool updateCalibrateButton = true);
     void sensorRemoved(const QDBusObjectPath &sensorPath, bool updateCalibrateButton = true);
 
