@@ -19,6 +19,8 @@
 
 #include "Profile.h"
 
+#include <QCryptographicHash>
+
 #include <KGlobal>
 #include <KLocale>
 #include <KFile>
@@ -50,6 +52,11 @@ void Profile::setFilename(const QString &filename)
     }
 }
 
+QString Profile::errorMessage() const
+{
+    return m_errorMessage;
+}
+
 QColor Profile::convertXYZ(cmsCIEXYZ *cieXYZ)
 {
     typedef struct {
@@ -99,6 +106,8 @@ void Profile::parseProfile(const uint *data, size_t length)
     /* ensure we have the header */
     if (length < 0x84) {
         kWarning() << "profile was not valid (file size too small)";
+        m_errorMessage = i18n("file too small");
+        m_loaded = false;
         return;
     }
 
@@ -106,6 +115,8 @@ void Profile::parseProfile(const uint *data, size_t length)
     m_lcmsProfile = cmsOpenProfileFromMem(data, length);
     if (m_lcmsProfile == NULL) {
         kWarning() << "failed to load: not an ICC profile";
+        m_errorMessage = i18n("not an ICC profile");
+        m_loaded = false;
         return;
     }
 
@@ -176,37 +187,37 @@ void Profile::parseProfile(const uint *data, size_t length)
     color_space = cmsGetColorSpace(m_lcmsProfile);
     switch (color_space) {
     case cmsSigXYZData:
-        m_colorspace = i18n("XYZ");
+        m_colorspace = i18nc("colorspace", "XYZ");
         break;
     case cmsSigLabData:
-        m_colorspace = i18n("LAB");
+        m_colorspace = i18nc("colorspace", "LAB");
         break;
     case cmsSigLuvData:
-        m_colorspace = i18n("LUV");
+        m_colorspace = i18nc("colorspace", "LUV");
         break;
     case cmsSigYCbCrData:
-        m_colorspace = i18n("YCbCr");
+        m_colorspace = i18nc("colorspace", "YCbCr");
         break;
     case cmsSigYxyData:
-        m_colorspace = i18n("Yxy");
+        m_colorspace = i18nc("colorspace", "Yxy");
         break;
     case cmsSigRgbData:
-        m_colorspace = i18n("RGB");
+        m_colorspace = i18nc("colorspace", "RGB");
         break;
     case cmsSigGrayData:
-        m_colorspace = i18n("Gray");
+        m_colorspace = i18nc("colorspace", "Gray");
         break;
     case cmsSigHsvData:
-        m_colorspace = i18n("HSV");
+        m_colorspace = i18nc("colorspace", "HSV");
         break;
     case cmsSigCmykData:
-        m_colorspace = i18n("CMYK");
+        m_colorspace = i18nc("colorspace", "CMYK");
         break;
     case cmsSigCmyData:
-        m_colorspace = i18n("CMY");
+        m_colorspace = i18nc("colorspace", "CMY");
         break;
     default:
-        m_colorspace = i18n("Unknown");
+        m_colorspace = i18nc("colorspace", "Unknown");
     }
 
     /* get the illuminants from the primaries */
@@ -319,6 +330,11 @@ void Profile::parseProfile(const uint *data, size_t length)
 //    /* success */
 //    ret = TRUE;
 
+    QCryptographicHash hash(QCryptographicHash::Md5);
+    hash.addData(reinterpret_cast<const char *>(data), length);
+    m_checksum = hash.result().toHex();
+    kDebug() << "checksum" << m_checksum;
+
 //    /* generate and set checksum */
 //    checksum = g_compute_checksum_for_data (G_CHECKSUM_MD5, (const guchar *) data, length);
 //    gcm_profile_set_checksum (profile, checksum);
@@ -342,21 +358,21 @@ QString Profile::kindString() const
 {
     switch (kind()) {
     case KindInputDevice:
-        return i18n("Input device");
+        return i18nc("profile kind", "Input device");
     case KindDisplayDevice:
-        return i18n("Display device");
+        return i18nc("profile kind", "Display device");
     case KindOutputDevice:
-        return i18n("Output device");
+        return i18nc("profile kind", "Output device");
     case KindDeviceLink:
-        return i18n("Devicelink");
+        return i18nc("profile kind", "Devicelink");
     case KindColorspaceConversion:
-        return i18n("Colorspace conversion");
+        return i18nc("profile kind", "Colorspace conversion");
     case KindAbstract:
-        return i18n("Abstract");
+        return i18nc("profile kind", "Abstract");
     case KindNamedColor:
-        return i18n("Named color");
+        return i18nc("profile kind", "Named color");
     default:
-        return i18n("Unknown");
+        return i18nc("profile kind", "Unknown");
     }
 }
 
