@@ -20,45 +20,29 @@
 #ifndef COLORD_H
 #define COLORD_H
 
-#include <KDEDModule>
-
 #include "Output.h"
 
-#include <KDirWatch>
-
-#include <QVariantList>
-#include <QFileInfo>
-#include <QtDBus/QDBusConnection>
-#include <QtDBus/QDBusInterface>
-#include <QtDBus/QDBusReply>
+#include <KDEDModule>
 
 extern "C"
 {
     #include <fixx11h.h>
     #include <X11/Xatom.h>
-    #include <X11/Xlib.h>
-    #define INT8 _X11INT8
-    #define INT32 _X11INT32
-    #include <X11/Xproto.h>
-    #undef INT8
-    #undef INT32
     #include <X11/extensions/Xrandr.h>
 }
 
-typedef QList<int> ScreenList;
-typedef QMap<QString, QString>  StringStringMap;
-
-class Edid;
+class CdInterface;
 class XEventHandler;
+class ProfilesWatcher;
 class ColorD : public KDEDModule
 {
     Q_OBJECT
 public:
     ColorD(QObject *parent, const QVariantList &args);
     ~ColorD();
-
-    void scanHomeDirectory();
     void reset();
+    int eventBase() const;
+    bool isValid() const;
 
 private slots:
     void init();
@@ -68,41 +52,32 @@ private slots:
     void deviceAdded(const QDBusObjectPath &objectPath);
     void deviceChanged(const QDBusObjectPath &objectPath);
 
-    void addProfile(const QString &filename);
-    void removeProfile(const QString &filename);
-
     void serviceOwnerChanged(const QString &serviceName, const QString &oldOwner, const QString &newOwner);
 
 private:
     quint8* readEdidData(RROutput output, size_t &len);
-    void connectToDisplay();
+    XRRScreenResources *connectToDisplay();
     void connectToColorD();
-    void addOutput(Output &output);
-    void outputChanged(Output &output);
-    void removeOutput(const Output &output);
-    void addProfile(const QFileInfo &fileInfo);
-    void addProfileToDevice(const QDBusObjectPath &profilePath, const QDBusObjectPath &devicePath);
-    void addEdidProfileToDevice(Output &output);
-    StringStringMap getProfileMetadata(const QDBusObjectPath &profilePath);
+    void addOutput(const Output::Ptr &output);
+    void outputChanged(const Output::Ptr &output);
+    void removeOutput(const Output::Ptr &output);
+    void addEdidProfileToDevice(const Output::Ptr &output);
+    CdStringMap getProfileMetadata(const QDBusObjectPath &profilePath);
     QString profilesPath() const;
 
-    KDirWatch *m_dirWatch;
-    QList<Output> m_connectedOutputs;
+    Output::List m_connectedOutputs;
 
     Display *m_dpy;
-    XEventHandler *m_eventHandler;
     XRRScreenResources *m_resources;
     Window m_root;
-    
-    bool m_valid;
+
     QString m_errorCode;
-    QString m_version;
 
     bool m_has_1_3;
-    int m_eventBase;
     int m_errorBase;
+    XEventHandler *m_x11EventHandler;
+    ProfilesWatcher *m_profilesWatcher;
+    CdInterface *m_cdInterface;
 };
 
-Q_DECLARE_METATYPE(StringStringMap)
-
 #endif // COLORD_H
