@@ -23,14 +23,14 @@
 
 #include <KDebug>
 
-#include <X11/Xatom.h>
-
 #define RR_CONNECTOR_TYPE_PANEL "Panel"
 
 Output::Output(RROutput output, XRRScreenResources *resources) :
     m_output(output),
     m_resources(resources),
-    m_connected(false)
+    m_interface(0),
+    m_connected(false),
+    m_isLaptop(false)
 {
     XRROutputInfo *info;
     info = XRRGetOutputInfo(QX11Info::display(), m_resources, m_output);
@@ -49,6 +49,26 @@ Output::Output(RROutput output, XRRScreenResources *resources) :
     m_crtc = info->crtc;
 
     XRRFreeOutputInfo(info);
+
+    // The ConnectorType property is present in RANDR 1.3 and greater
+    if (connectorType() == QLatin1String(RR_CONNECTOR_TYPE_PANEL)) {
+        m_isLaptop = true;
+    } else if (m_name.contains(QLatin1String("lvds"), Qt::CaseInsensitive) ||
+               // Most drivers use an "LVDS" prefix...
+               m_name.contains(QLatin1String("LCD"), Qt::CaseInsensitive) ||
+               // ... but fglrx uses "LCD" in some versions.  Shoot me now, kthxbye.
+               m_name.contains(QLatin1String("eDP"), Qt::CaseInsensitive)
+               /* eDP is for internal laptop panel connections */) {
+        // Older versions of RANDR - this is a best guess,
+        // as @#$% RANDR doesn't have standard output names,
+        // so drivers can use whatever they like.
+        m_isLaptop = true;
+    }
+}
+
+Output::~Output()
+{
+    delete m_interface;
 }
 
 bool Output::connected() const
@@ -56,62 +76,9 @@ bool Output::connected() const
     return m_connected;
 }
 
-static QString getConnectorTypeString(Display *dpy, RROutput output)
-{
-    unsigned char *prop;
-    int actual_format;
-    unsigned long nitems, bytes_after;
-    Atom actual_type;
-    Atom connector_type;
-    Atom connector_type_atom = XInternAtom(dpy, "ConnectorType", false);
-    char *connector_type_str;
-    QString result;
-
-    XRRGetOutputProperty(dpy, output, connector_type_atom,
-                         0, 100, false, false,
-                         AnyPropertyType,
-                         &actual_type, &actual_format,
-                         &nitems, &bytes_after, &prop);
-    if (!(actual_type == XA_ATOM && actual_format == 32 && nitems == 1)) {
-        XFree(prop);
-        return result;
-    }
-
-    connector_type = *((Atom *) prop);
-
-    connector_type_str = XGetAtomName(dpy, connector_type);
-    if (connector_type_str) {
-        result = connector_type_str;
-        XFree(connector_type_str);
-    }
-
-    XFree (prop);
-
-    return result;
-}
-
 bool Output::isLaptop() const
 {
-    // The ConnectorType property is present in RANDR 1.3 and greater
-    QString connectorType = getConnectorTypeString(QX11Info::display(), m_output);
-    kDebug() << connectorType;
-    if (connectorType == QLatin1String(RR_CONNECTOR_TYPE_PANEL)) {
-        return true;
-    }
-
-    // Older versions of RANDR - this is a best guess, as @#$% RANDR doesn't have standard output names,
-    // so drivers can use whatever they like.
-
-    // Most drivers use an "LVDS" prefix...
-    if (m_name.contains(QLatin1String("lvds"), Qt::CaseInsensitive) ||
-            // ... but fglrx uses "LCD" in some versions.  Shoot me now, kthxbye.
-        m_name.contains(QLatin1String("LCD"), Qt::CaseInsensitive) ||
-            // eDP is for internal laptop panel connections
-        m_name.contains(QLatin1String("eDP"), Qt::CaseInsensitive)) {
-        return true;
-    }
-
-    return false;
+    return m_isLaptop;
 }
 
 bool Output::isPrimary(bool hasXRandR13, Window root) const
@@ -135,7 +102,20 @@ QString Output::id() const
 
 void Output::setPath(const QDBusObjectPath &path)
 {
+    if (m_interface && m_interface->path() == path.path()) {
+        return;
+    }
     m_path = path;
+
+    delete m_interface;
+    m_interface = new CdDeviceInterface(QLatin1String("org.freedesktop.ColorManager"),
+                                        path.path(),
+                                        QDBusConnection::systemBus());
+    if (!m_interface->isValid()) {
+        kWarning() << "Invalid interface" << path.path() << m_interface->lastError().message();
+        delete m_interface;
+        m_interface = 0;
+    }
 }
 
 QDBusObjectPath Output::path() const
@@ -143,6 +123,11 @@ QDBusObjectPath Output::path() const
     return m_path;
 }
 
+CdDeviceInterface *Output::interface()
+{
+    return m_interface;
+}
+
 RRCrtc Output::crtc() const
 {
     return m_crtc;
@@ -153,7 +138,7 @@ RROutput Output::output() const
     return m_output;
 }
 
-int Output::getGammaSize()
+int Output::getGammaSize() const
 {
     // The gama size of this output
     return XRRGetCrtcGammaSize(QX11Info::display(), m_crtc);
@@ -170,9 +155,11 @@ Edid Output::readEdidData()
     size_t size;
     const quint8 *data;
     data = readEdidData(size);
-    if (data == 0) {
-        kWarning() << "unable to get EDID for output";
-        return Edid();
+    if (data == NULL) {
+        kWarning() << "Unable to get EDID for output" << name();
+        Edid ret;
+        m_id = ret.deviceId(name());
+        return ret;
     }
 
     // With EDID data we can parse our info
@@ -189,6 +176,40 @@ QString Output::edidHash() const
     return m_edidHash;
 }
 
+QString Output::connectorType() const
+{
+    unsigned char *prop;
+    int actual_format;
+    unsigned long nitems, bytes_after;
+    Atom actual_type;
+    Atom connector_type;
+    Atom connector_type_atom = XInternAtom(QX11Info::display(), "ConnectorType", false);
+    char *connector_type_str;
+    QString result;
+
+    XRRGetOutputProperty(QX11Info::display(), m_output, connector_type_atom,
+                         0, 100, false, false,
+                         AnyPropertyType,
+                         &actual_type, &actual_format,
+                         &nitems, &bytes_after, &prop);
+    if (!(actual_type == XA_ATOM && actual_format == 32 && nitems == 1)) {
+        XFree(prop);
+        return result;
+    }
+
+    connector_type = *((Atom *) prop);
+
+    connector_type_str = XGetAtomName(QX11Info::display(), connector_type);
+    if (connector_type_str) {
+        result = connector_type_str;
+        XFree(connector_type_str);
+    }
+
+    XFree (prop);
+
+    return result;
+}
+
 /* This is what gnome-desktop does */
 static quint8* getProperty(Display *dpy,
                            RROutput output,
@@ -225,10 +246,6 @@ quint8* Output::readEdidData(size_t &len)
 
     edid_atom = XInternAtom(QX11Info::display(), RR_PROPERTY_RANDR_EDID, false);
     result = getProperty(QX11Info::display(), m_output, edid_atom, len);
-    if (result == NULL) {
-        edid_atom = XInternAtom(QX11Info::display(), "EDID_DATA", false);
-        result = getProperty(QX11Info::display(), m_output, edid_atom, len);
-    }
 
     if (result) {
         if (len % 128 == 0) {
@@ -239,7 +256,7 @@ quint8* Output::readEdidData(size_t &len)
         }
     }
 
-    return 0;
+    return NULL;
 }
 
 bool Output::operator==(const Output &output) const
