@@ -22,17 +22,13 @@
 
 #include <QDBusObjectPath>
 #include <QTextStream>
+#include "CdDeviceInterface.h"
 
 #include "Edid.h"
 
 extern "C"
 {
-    #include <X11/Xlib.h>
-    #define INT8 _X11INT8
-    #define INT32 _X11INT32
-    #include <X11/Xproto.h>
-    #undef INT8
-    #undef INT32
+    #include <X11/Xatom.h>
     #include <X11/extensions/Xrandr.h>
 }
 
@@ -40,7 +36,10 @@ class Output
 {
     Q_GADGET
 public:
+    typedef QSharedPointer<Output> Ptr;
+    typedef QList<Ptr> List;
     Output(RROutput output, XRRScreenResources *resources);
+    ~Output();
 
     bool connected() const;
     bool isLaptop() const;
@@ -49,13 +48,15 @@ public:
     QString id() const;
     void setPath(const QDBusObjectPath &path);
     QDBusObjectPath path() const;
+    CdDeviceInterface* interface();
     RRCrtc crtc() const;
     RROutput output() const;
-    int getGammaSize();
+    int getGammaSize() const;
     void setGamma(XRRCrtcGamma *gamma);
 
     Edid readEdidData();
     QString edidHash() const;
+    QString connectorType() const;
 
     bool operator==(const Output &output) const;
 
@@ -69,9 +70,11 @@ private:
     XRRScreenResources *m_resources;
     QString m_edidHash;
     QString m_id;
+    CdDeviceInterface *m_interface;
     QDBusObjectPath m_path;
 
     bool m_connected;
+    bool m_isLaptop;
     QString m_name;
     RRCrtc m_crtc;
 };
